var CLOSE_CODES, DEFAULT_STYLES, EventEmitter, Logger, Node, TagNode, TextNode,
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

EventEmitter = require('events').EventEmitter;

DEFAULT_STYLES = {
  "default": [],
  bold: ['\x1B[1m'],
  italic: ['\x1B[3m'],
  underline: ['\x1B[4m'],
  inverse: ['\x1B[7m'],
  strikethrough: ['\x1B[9m'],
  white: ['\x1B[37m'],
  grey: ['\x1B[90m'],
  black: ['\x1B[30m'],
  blue: ['\x1B[34m'],
  cyan: ['\x1B[36m'],
  green: ['\x1B[32m'],
  magenta: ['\x1B[35m'],
  red: ['\x1B[31m'],
  yellow: ['\x1B[33m'],
  whiteBG: ['\x1B[47m'],
  greyBG: ['\x1B[49;5;8m'],
  blackBG: ['\x1B[40m'],
  blueBG: ['\x1B[44m'],
  cyanBG: ['\x1B[46m'],
  greenBG: ['\x1B[42m'],
  magentaBG: ['\x1B[45m'],
  redBG: ['\x1B[41m'],
  yellowBG: ['\x1B[43m']
};

CLOSE_CODES = {
  '\x1B[1m': '\x1B[22m',
  '\x1B[3m': '\x1B[23m',
  '\x1B[4m': '\x1B[24m',
  '\x1B[7m': '\x1B[27m',
  '\x1B[9m': '\x1B[29m',
  '\x1B[37m': '\x1B[39m',
  '\x1B[90m': '\x1B[39m',
  '\x1B[30m': '\x1B[39m',
  '\x1B[34m': '\x1B[39m',
  '\x1B[36m': '\x1B[39m',
  '\x1B[32m': '\x1B[39m',
  '\x1B[35m': '\x1B[39m',
  '\x1B[31m': '\x1B[39m',
  '\x1B[33m': '\x1B[39m',
  '\x1B[47m': '\x1B[49m',
  '\x1B[49;5;8m': '\x1B[49m',
  '\x1B[40m': '\x1B[49m',
  '\x1B[44m': '\x1B[49m',
  '\x1B[46m': '\x1B[49m',
  '\x1B[42m': '\x1B[49m',
  '\x1B[45m': '\x1B[49m',
  '\x1B[41m': '\x1B[49m',
  '\x1B[43m': '\x1B[49m'
};

Node = (function(_super) {
  __extends(Node, _super);

  function Node(_arg) {
    this.type = _arg.type, this.open = _arg.open, this.close = _arg.close;
    Node.__super__.constructor.call(this);
    this.type || (this.type = 'root');
    this.children || (this.children = []);
    this.open || (this.open = []);
    this.close || (this.close = []);
    this.length || (this.length = 0);
  }

  Node.prototype.append = function(child) {
    var _this = this;
    this.length += child.length;
    this.children.push(child);
    this.emit('append', child);
    return child.on('append', function(child) {
      _this.length += child.length;
      return _this.emit('append', child);
    });
  };

  Node.prototype.appendCodes = function(codes) {
    var code, _i, _len, _results;
    _results = [];
    for (_i = 0, _len = codes.length; _i < _len; _i++) {
      code = codes[_i];
      this.open.push(code);
      _results.push(this.close.unshift(CLOSE_CODES[code]));
    }
    return _results;
  };

  Node.prototype.style = function() {
    return this._style().join('');
  };

  Node.prototype._style = function() {
    throw new Error('Not implemented');
  };

  Node.prototype.copy = function() {
    throw new Error('Not implemented');
  };

  Node.prototype.slice = function(start, end) {
    var copy;
    if (start == null) {
      start = 0;
    }
    if (end == null) {
      end = Infinity;
    }
    copy = this.copy();
    copy.splice(start, end);
    return copy;
  };

  Node.prototype.splice = function() {
    throw new Error('Not implemented');
  };

  return Node;

})(EventEmitter);

TextNode = (function(_super) {
  __extends(TextNode, _super);

  function TextNode(value, opts) {
    this.value = value;
    if (opts == null) {
      opts = {};
    }
    opts.type = 'text';
    TextNode.__super__.constructor.call(this, opts);
    this.length = value.length;
  }

  TextNode.prototype._style = function() {
    return [unescape(this.value)];
  };

  TextNode.prototype.copy = function() {
    var copy;
    copy = new TextNode(this.value, {
      type: this.type,
      open: this.open.slice(0),
      close: this.close.slice(0)
    });
    return copy;
  };

  TextNode.prototype.splice = function(start, end) {
    if (start == null) {
      start = 0;
    }
    if (end == null) {
      end = Infinity;
    }
    this.value = this.value.slice(start, end);
    this.length = this.value.length;
    return this;
  };

  return TextNode;

})(Node);

TagNode = (function(_super) {
  __extends(TagNode, _super);

  function TagNode(name, opts) {
    this.name = name;
    if (opts == null) {
      opts = {};
    }
    opts.type = 'tag';
    TagNode.__super__.constructor.call(this, opts);
  }

  TagNode.prototype._style = function() {
    var childNode, code, output, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2;
    if (this.children.length === 0) {
      return;
    }
    output = [];
    _ref = this.open;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      code = _ref[_i];
      output.push(code);
    }
    _ref1 = this.children;
    for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
      childNode = _ref1[_j];
      output = output.concat(childNode._style());
      output = output.concat(this.open);
    }
    _ref2 = this.close;
    for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
      code = _ref2[_k];
      output.push(code);
    }
    return output;
  };

  TagNode.prototype.copy = function() {
    var child, copy, _i, _len, _ref;
    copy = new TagNode(this.tag, {
      type: this.type,
      open: this.open.slice(0),
      close: this.close.slice(0)
    });
    _ref = this.children;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      child = _ref[_i];
      copy.append(child.copy());
    }
    return copy;
  };

  TagNode.prototype.splice = function(start, end) {
    var child, i, index, newIndex, _i, _len, _ref;
    if (start == null) {
      start = 0;
    }
    if (end == null) {
      end = Infinity;
    }
    index = 0;
    _ref = this.children;
    for (i = _i = 0, _len = _ref.length; _i < _len; i = ++_i) {
      child = _ref[i];
      newIndex = index + child.length;
      if (index + child.length < start) {
        this.children.shift();
        this.length -= child.length;
      } else {
        if (newIndex >= end) {
          this.length -= child.length;
          child.splice(0, end - index);
          this.length += child.length;
        }
        if (start > index && start < newIndex) {
          this.length -= child.length;
          child.splice(start - index);
          this.length += child.length;
        }
        if (newIndex === end) {
          break;
        }
      }
    }
    return this;
  };

  return TagNode;

})(Node);

Logger = (function() {
  function Logger(options) {
    var name, value, _base;
    if (options == null) {
      options = {};
    }
    this.format = __bind(this.format, this);
    this.emit = __bind(this.emit, this);
    this.log = __bind(this.log, this);
    this.options = {};
    for (name in options) {
      if (!__hasProp.call(options, name)) continue;
      value = options[name];
      this.options[name] = value;
    }
    this._styles = {};
    for (name in DEFAULT_STYLES) {
      value = DEFAULT_STYLES[name];
      this._styles[name] = value.slice(0);
    }
    this._events = {};
    this._modes = {};
    (_base = this.options).tab || (_base.tab = 6);
    if (!this.options.gutter && this.options.gutter !== '') {
      this.options.gutter = ' - ';
    }
    this.options.gutter = this.parse(this.options.gutter);
  }

  Logger.prototype.log = function(data, options) {
    var colors, depth, showHidden, _ref;
    if (options == null) {
      options = {};
    }
    colors = options.colors, depth = options.depth, showHidden = options.showHidden;
    if (colors == null) {
      colors = true;
    }
    if (typeof data === 'string') {
      console.log(this.format(data));
    } else if ((_ref = typeof data) === 'object' || _ref === 'number') {
      console.log(util.inspect(data, {
        depth: depth,
        showHidden: showHidden,
        colors: colors
      }));
    } else {
      console.log(data);
    }
    return this;
  };

  Logger.prototype.emit = function(event, data) {
    var dataLine, dataLines, eventLine, eventLines, gutter, gutterSpace, i, line, output, tabsSpace, _i, _ref;
    if ((this._mode != null) && this._modes[this._mode] !== true && (this._mode === false || __indexOf.call(this._modes[this._mode], event) < 0)) {
      return;
    }
    event = this._events[event] || (this.parse(event));
    data = this.format(data);
    output = [];
    dataLines = data.split('\n');
    eventLines = (function() {
      var _i, _ref, _results;
      _results = [];
      for (i = _i = 0, _ref = Math.ceil(event.length / this.options.tab); 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
        _results.push(event.slice(i * this.options.tab, (i + 1) * this.options.tab));
      }
      return _results;
    }).call(this);
    tabsSpace = (new Array(this.options.tab + 1)).join(' ');
    gutterSpace = (new Array(this.options.gutter.length + 1)).join(' ');
    gutter = this.options.gutter.style();
    for (i = _i = 0, _ref = Math.max(dataLines.length, eventLines.length); 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
      line = [];
      eventLine = eventLines[i];
      dataLine = dataLines[i];
      if (eventLine) {
        line.push((new Array(this.options.tab - eventLine.length + 1)).join(' '));
        line.push(eventLine.style());
      } else {
        line.push(tabsSpace);
      }
      line.push(i === 0 ? gutter : gutterSpace);
      line.push(dataLine);
      output.push(line.join(''));
    }
    console.log(output.join('\n'));
    return this;
  };

  Logger.prototype.format = function(str) {
    return (this.parse(str)).style();
  };

  Logger.prototype.parse = function(str) {
    var code, index, lastIndex, match, matchesLength, node, open, output, reg, root, style, tag, tree, _i, _len, _ref, _ref1;
    reg = /<\/?[a-zA-Z0-9-]*>/g;
    output = [];
    lastIndex = 0;
    matchesLength = 0;
    tree = [];
    root = new TagNode('root', {
      open: this._styles["default"].slice(0) || [],
      close: (function() {
        var _i, _ref, _results;
        _ref = this._styles["default"];
        _results = [];
        for (_i = _ref.length - 1; _i >= 0; _i += -1) {
          code = _ref[_i];
          _results.push(CLOSE_CODES[code]);
        }
        return _results;
      }).call(this)
    });
    tree.unshift(root);
    while (match = reg.exec(str)) {
      _ref = match, match = _ref[0], index = _ref.index;
      if (str.slice(lastIndex, index) !== '') {
        tree[0].append(new TextNode(str.slice(lastIndex, index)));
      }
      lastIndex = index + match.length;
      open = match[1] !== '/';
      tag = open ? match.slice(1, -1) : match.slice(2, -1);
      if (open) {
        node = new TagNode(tag);
        tree[0].append(node);
        if ((tag.indexOf('-')) !== -1) {
          _ref1 = tag.split('-');
          for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
            style = _ref1[_i];
            if (!(style in this._styles)) {
              throw new Error("Non-existent style " + style + " at index " + index);
            }
            node.appendCodes(this._styles[style]);
          }
        } else {
          if (!(tag in this._styles)) {
            throw new Error("Non-existent style " + tag + " at index " + index);
          }
          node.appendCodes(this._styles[tag]);
        }
        tree.unshift(node);
      } else {
        node = tree.shift();
        if (node.name !== tag && match !== '</>') {
          throw new Error("Expected </" + node.name + "> or </> closing tag, received </" + tag + "> instead at index " + index);
        }
      }
    }
    tree[0].append(new TextNode(str.slice(lastIndex)));
    return root;
  };

  Logger.prototype.setMode = function(mode) {
    if (!(mode in this._modes)) {
      throw new Error("Unknown mode (" + mode + ")");
    }
    this._mode = mode;
    return this;
  };

  Logger.prototype.registerMode = function(name, events) {
    if (name === 'default') {
      this._mode || (this._mode = name);
    }
    this._modes[name] = events;
    return this;
  };

  Logger.prototype.registerEvent = function(name, display) {
    this._events[name] = this.parse(display);
    return this;
  };

  Logger.prototype.registerStyle = function(name, styles) {
    var style, _ref;
    this._styles[name] = (_ref = []).concat.apply(_ref, (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = styles.length; _i < _len; _i++) {
        style = styles[_i];
        _results.push(this._styles[style]);
      }
      return _results;
    }).call(this));
    return this;
  };

  return Logger;

})();

Logger.Node = Node;

Logger.TagNode = TagNode;

Logger.TextNode = TextNode;

module.exports = Logger;
