var ADDRTYPE, AUTH, CMD, Consumable, EventEmitter, RSV, STATUS, Server, VER, net, u, _ref,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
  __slice = [].slice;

net = require('net');

EventEmitter = require('events').EventEmitter;

u = require('../utilities');

Consumable = require('../consumable');

_ref = require('./const'), VER = _ref.VER, AUTH = _ref.AUTH, CMD = _ref.CMD, ADDRTYPE = _ref.ADDRTYPE, STATUS = _ref.STATUS, RSV = _ref.RSV;

module.exports = Server = (function(_super) {
  __extends(Server, _super);

  Server.prototype.methods = ['getConnections', 'listen', 'close', 'address', 'unref', 'ref'];

  function Server() {
    this._requestHandler = __bind(this._requestHandler, this);
    this._authenticationHandler = __bind(this._authenticationHandler, this);
    this._awaitData = __bind(this._awaitData, this);
    var method, _i, _len, _ref1,
      _this = this;
    Server.__super__.constructor.apply(this, arguments);
    this.server = net.createServer(function(clientConnection) {
      return _this._awaitData(clientConnection, _this._authenticationHandler);
    }).on('listening', function() {
      return _this.emit('listening');
    }).on('close', function() {
      return _this.emit('close');
    }).on('error', function(err) {
      return _this.emit('error', err);
    });
    _ref1 = this.methods;
    for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
      method = _ref1[_i];
      this[method] = this.server[method].bind(this.server);
    }
  }

  Server.prototype._awaitData = function(clientConnection, callback) {
    var _this = this;
    return clientConnection.once('data', function(chunk) {
      return callback(clientConnection, new Consumable(chunk));
    });
  };

  Server.prototype._authenticationHandler = function(clientConnection, data) {
    var auth, i, methods, nmethods, version, _ref1;
    version = data.byte();
    if (version !== VER.SOCKS5) {
      this.emit('clientError', new Error("Invalid client version (" + version + ")"), data.buffer);
      clientConnection.destroy();
      return;
    }
    nmethods = data.byte();
    methods = (function() {
      var _i, _results;
      _results = [];
      for (i = _i = 0; 0 <= nmethods ? _i < nmethods : _i > nmethods; i = 0 <= nmethods ? ++_i : --_i) {
        _results.push(data.byte());
      }
      return _results;
    })();
    if (_ref1 = AUTH.NOAUTH, __indexOf.call(methods, _ref1) >= 0) {
      auth = AUTH.NOAUTH;
    } else {
      auth = AUTH.NO_ACCEPTABLE_METHOD;
      this.emit('clientError', new Error("No acceptable methods (" + (methods.join(', ')) + ")"), data.buffer);
    }
    clientConnection.write(new Buffer([VER.SOCKS5, auth]));
    return this._awaitData(clientConnection, this._requestHandler);
  };

  Server.prototype._requestHandler = function(clientConnection, data) {
    var addrType, command, host, localAddress, localPort, port, rsv, version;
    version = data.byte();
    if (version !== VER.SOCKS5) {
      clientConnection.write(new Buffer([VER.SOCKS5, STATUS.SERVER_FAILURE]));
      this.emit('clientError', new Error("Unsupported version (" + version + ")"), data.buffer);
      clientConnection.end();
      return;
    }
    command = data.byte();
    if (command !== CMD.CONNECT) {
      clientConnection.write(new Buffer([VER.SOCKS5, STATUS.COMMAND_NOT_SUPPORTED]));
      this.emit('clientError', new Error("Unsupported command (" + command + ")"), data.buffer);
      clientConnection.end();
      return;
    }
    rsv = data.byte();
    addrType = data.byte();
    if (addrType !== ADDRTYPE.IPV4 && addrType !== ADDRTYPE.IPV6 && addrType !== ADDRTYPE.DOMAIN) {
      clientConnection.write(new Buffer([VER.SOCKS5, STATUS.ADDRTYPE_NOT_SUPPORTED]));
      this.emit('clientError', new Error("Unsupported address type (" + addrType + ")"), data.buffer);
      clientConnection.end();
      return;
    }
    host = (function() {
      switch (addrType) {
        case ADDRTYPE.IPV4:
          return u.formatIPv4(data.read(4));
        case ADDRTYPE.IPV6:
          return u.formatIPv6(data.read(16));
        case ADDRTYPE.DOMAIN:
          return data.char(data.byte());
      }
    })();
    port = data.uint16();
    localAddress = clientConnection.localAddress, localPort = clientConnection.localPort;
    return this.emit('request', clientConnection, host, port, function(status) {
      if (status && status !== STATUS.SUCCESS) {
        clientConnection.write(new Buffer([VER.SOCKS5, status]));
        clientConnection.end();
        return;
      }
      return clientConnection.write(new Buffer([VER.SOCKS5, STATUS.SUCCESS, RSV, ADDRTYPE.IPV4].concat(__slice.call(u.parseIPv4(localAddress)), __slice.call(u.toUInt16BE(localPort)))));
    });
  };

  return Server;

})(EventEmitter);
