exports.toUInt16LE = function(value) {
  return [value & 0x00ff, (value & 0xff00) >>> 8];
};

exports.toUInt16BE = function(value) {
  return [(value & 0xff00) >>> 8, value & 0x00ff];
};

exports.toUInt32LE = function(value) {
  return [value & 0xff, (value >>> 8) & 0xff, (value >>> 16) & 0xff, (value >>> 24) & 0xff];
};

exports.toUInt32BE = function(value) {
  return [(value >>> 24) & 0xff, (value >>> 16) & 0xff, (value >>> 8) & 0xff, value & 0xff];
};

exports.formatIPv4 = function(value) {
  return Array.prototype.join.call(value, '.');
};

exports.formatIPv6 = function(value) {
  var i;
  if (!Buffer.isBuffer(value)) {
    value = new Buffer(value);
  }
  return ((function() {
    var _i, _ref, _results;
    _results = [];
    for (i = _i = 0, _ref = value.length; _i < _ref; i = _i += 2) {
      _results.push("" + (value.slice(i, +(i + 1) + 1 || 9e9).toString('hex')));
    }
    return _results;
  })()).join(':');
};

exports.parseIPv4 = function(value) {
  return (value.split('.')).map(function(v) {
    return parseInt(v, 10);
  });
};

exports.parseIPv6 = function(value) {
  var offset, output, part, parts, _i, _len, _ref;
  output = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
  offset = 0;
  parts = value.split(':');
  for (_i = 0, _len = parts.length; _i < _len; _i++) {
    part = parts[_i];
    if (part) {
      [].splice.apply(output, [offset, (offset + 2) - offset].concat(_ref = exports.toUInt16BE(parseInt(part, 16)))), _ref;
      offset += 2;
    } else {
      offset += 16 - (parts.length * 2 - offset);
    }
  }
  return output;
};
